function density = dndensity1(seq,varargin)
%DNDENSITY plots the density of di-nucleotides in a sequence.
%
%   DNDENSITY1(SEQ) plots the density of 16 di-nucleotides in sequence
%   SEQ.
%
%   DENSITY = DNDENSITY1(SEQ) returns a structure of the density of
%   16 di-nucleotides AA, AC, AG, AT, CA, CC, CG, CT, GA, GC, GG, GT,
%   TA, TC, TG, TT.
%
%   DNDENSITY1(...,'DINUCLEOTIDE',DN) plots the density of the di-nucleotide
%   which is specified by DN. DN can be a list of di-nucleotides needed to
%   plot. For example DN={'AC','TG','GT'}.
%
%   DNDENSITY1(...,'WINDOW',L) uses a window of length L for the density
%   calculation. The default value is length(SEQ)/20.
%
%   Example:
%
%       % Create a random sequence and analyze the nucleotide density.
%       seq = randseq(240)
%       dndensity(seq)
%
%   See also GENSIGN, DIMERCOUNT, NTDENSITY, FILTER


window = ceil(length(seq)/20);
dinucleotide={'AT','CG'};
seq=upper(seq);

if nargin > 1
    if rem(nargin,2)== 0
        error('Bioinfo:IncorrectNumberOfArguments',...
            'Incorrect number of arguments to %s.',mfilename);
    end
    okargs = {'dinucleotide','window'};
    for j=1:2:nargin-2
        pname = varargin{j};
        pval = varargin{j+1};
        k = strmatch(lower(pname), okargs);
        if isempty(k)
            error('Bioinfo:UnknownParameterName',...
                'Unknown parameter name: %s.',pname);
        elseif length(k)>1
            error('Bioinfo:AmbiguousParameterName',...
                'Ambiguous parameter name: %s.',pname);
        else
            switch(k)
                case 1   %dinucleotide
                    dinucleotide=pval;
                case 2   % window
                    window = pval;
            end
        end
    end
end

DINUC={'AA','AC','AG','AT','CA','CC','CG','CT','GA','GC','GG','GT','TA','TC','TG','TT'};
for i=1:length(seq)-1
    tmp={[seq(i),seq(i+1)]};
    for j=1:16
        if isequal(tmp,DINUC(j))  tmp1(i)=j; end
    end
end

% Pad the end of the sequence to accomodate the window size
seq = [tmp1 zeros(1,window-1)];
len = length(seq);

% Calculate nucleotide densities with a moving window
b = 1/window*ones(1,window);
a = 1;

for i=1:16
s(i,:) = filter(b,a,seq==i);
end

% Deal with the fact that filter starts and ends with a series of zeros.
for i=1:16
    s(i,1:window-1) = s(i,1:window-1) * window./(1:window-1);
    s(i,len-window+2:len) = s(i,len-window+2:len) * window./(window-1:-1:1);
end

% Reduce the resulting vector to the correct subset
s=s(:,1+floor(window/2):len-floor(window/2));

% Detect the index of di-nucleotides which need to be plotted
numdn=length(dinucleotide); 
for i=1:numdn
    for j=1:16
        if isequal(DINUC(j),dinucleotide(i)) iddinuc(i)=j; end 
    end
end

% Create plot 
sm=s(iddinuc,:)';

% Here we reduce the original sampling rate for a sequence to a lower rate 
% if the sequence is longer than 500 Kb.

if length(sm(:,1))>5000
    for i=1:numdn
        r=round(length(sm(:,i))/1000);
        tmp(:,i)=decimate(sm(:,i),r);
    end
    sm=tmp;
end

plot(sm);
legend(dinucleotide);
title('Di-nucleotide density')

% Create outputs
if nargout > 0
    density.AA = s(1,:);
    density.AC = s(2,:);
    density.AG = s(3,:);
    density.AT = s(4,:);
    density.CA = s(5,:);
    density.CC = s(6,:);
    density.CG = s(7,:);
    density.CT = s(8,:);
    density.GA = s(9,:);
    density.GC = s(10,:);
    density.GG = s(11,:);
    density.GT = s(12,:);
    density.TA = s(13,:);
    density.TC = s(14,:);
    density.TG = s(15,:);
    density.TT = s(16,:);   
end


