function [orfOutput number]= seqorfs(seq,varargin)
%SEQORFS detemines the open reading frames (ORFs) in a sequence and in its 
%reverse complement.
%
%   orfOutput=SEQORFS(SEQ) returns a structure with start/stop positions,
%   length of ORF and which reading frame it is in. The reverse complement 
%   of the sequence is also cosidered. The Standard genetic code is used 
%   with start codon 'AUG' and stop codons 'UAA','UAG','UGA'. 
%
%   SEQORFS(...,'GENETICCODE',CODE) specifies the genetic code to be
%   used for finding open reading frames. Code can be either an ID or text
%   string. See help for geneticcode for the full list of supported IDs and
%   names.
%
%   SEQORFS(...,'MINIMUMLENGTH',L) sets the minimum number of codons
%   for an ORF to be considered valid. The default is 10.
%
%   [ORFOUTPUT NUMBER]=SEQORFS(SEQ,...) returns the number of the ORFs 
%   founded in the sequence and in its reverse complement.
%
%   Example:
%
%       mithocondria = getgenbank('NC_001807');
%       seqorfs(mithocondria.Sequence);
%
%   See also SEQSHOWORFS, CODONCOUNT, GENETICCODE, REGEXP.




% set up some defaults
theFrames = [1 2 3 -1 -2 -3];
seqLen = length(seq);
startCodons = 'A[TU]G|C[TU]G|[TU][TU]G';
stopCodons = '[TU]A[AG]|[TU]GA';
minLength = 10;
number=0;

% deal with the various inputs
if nargin > 1
    if rem(nargin,2) == 0
        error('Bioinfo:IncorrectNumberOfArguments',...
            'Incorrect number of arguments to %s.',mfilename);
    end
    okargs = {'geneticcode','minimumlength'};
    for j=1:2:nargin-2
        pname = varargin{j};
        pval = varargin{j+1};
        k = strmatch(lower(pname), okargs);
        if isempty(k)
            error('Bioinfo:UnknownParameterName',...
                'Unknown parameter name: %s.',pname);
        elseif length(k)>1
            error('Bioinfo:AmbiguousParameterName',...
                'Ambiguous parameter name: %s.',pname);
        else
            switch(k)
                case 1 % genetic code
                    try
                        gc = geneticcode(pval);
                    catch
                        if isnumeric(pval)
                            error('Bioinfo:BadGeneticCodeID','%s is not a valid Genetic Code ID',num2str(pval))
                        else
                            error('Bioinfo:BadGeneticCode','%s is not a valid Genetic Code',pval)
                        end
                    end
                    startCodons = gc.Starts{1};
                    for startcount = 2:length(gc.Starts)
                        startCodons = [startCodons '|' gc.Starts{startcount}];
                    end
                    startCodons = strrep(startCodons,'T','[TU]');

                    stopCodons = '';
                    map = 'ACGT';
                    for outer = 1:4
                        for mid = 1:4
                            for inner = 1:4
                                if gc.([map(outer) map(mid) map(inner)]) == '*'
                                    stopCodons = [stopCodons map(outer) map(mid) map(inner) '|'];
                                end
                            end
                        end
                    end
                    if stopCodons(end) == '|'
                        stopCodons(end) = '';
                    end
                    stopCodons = strrep(stopCodons,'T','[TU]');
              
                case 2 % minimum length
                    if isnumeric(pval) && pval > 0
                        minLength = pval;
                    else
                        error('Bioinfo:MinimumLengthNotPositiveInt',...
                            'MinimumLength must be a postive number.');
                    end
            end
        end
    end
end



% set up output
orfOutput.Start = [];
orfOutput.Stop = [];
orfOutput = repmat(orfOutput,1,6);

% set default startCodons
 startCodons = 'A[TU]G';

% we need to do reverse complement then make a copy

theFrames(theFrames<0) = 3-theFrames(theFrames<0);
rseq = seqrcomplement(seq);
rstarts = regexpi(rseq,startCodons);
rstops = regexpi(rseq,stopCodons);
fstarts = regexpi(seq,startCodons);
fstops = regexpi(seq,stopCodons);
fseq = seq;


for frame = theFrames
    
    if frame < 4
        starts = fstarts;
        stops = fstops;
        seq = fseq;
    else
        starts = rstarts;
        stops = rstops;
        seq = rseq;
    end
    

    % We now have to deal with figuring out which starts and stops are in
    % this frame.
    framestarts = [starts(rem(starts,3)==rem(frame,3)) inf];
    framestops = [stops(rem(stops,3)==rem(frame,3))  inf];
    framestartcodons = starts(rem(starts,3)==rem(frame,3));
    framestopcodons = stops(rem(stops,3)==rem(frame,3));


    % and we only care about stops after a start and starts after a stop
    % so we need to throw away any stops not in ORF and any starts inside
    % an already open reading frame.
    stopsAndStart = union(framestartcodons,framestopcodons);
    [dummy StartIndices ] = ismember(framestartcodons,stopsAndStart);
    [dummy StopIndices ] = ismember(framestopcodons,stopsAndStart);
    StartDiffs = diff([-1 StartIndices]);
    StopDiffs = diff([0 StopIndices]);

    frameStart = framestartcodons(StartDiffs ~= 1);
    frameStop = framestopcodons(StopDiffs ~= 1);

    % look for small ORFs and remove them
    if ~isempty(frameStop)
        smallORFs = find((frameStop-frameStart(1:numel(frameStop)))< minLength*3);
        frameStart(smallORFs) = [];
        frameStop(smallORFs) = [];
    end
    % clean up empty results
    if isempty(frameStart)
        frameStart = [];
    end
    if isempty(frameStop)
        frameStop = [];
    end
    
       
    % save the output
    orfOutput(frame).Start = frameStart(1:numel(frameStop));
    orfOutput(frame).Stop = frameStop;   
    if isempty(frameStop)
        orfOutput(frame).Length=[];
    else
        orfOutput(frame).Length=(frameStop-frameStart(1:numel(frameStop)));  
    end
    
    if frame < 4
        orfOutput(frame).Frame=frame*ones(1,numel(frameStop));
    else
        orfOutput(frame).Frame=(3-frame)*ones(1,numel(frameStop));
    end
    
    number=number+numel(frameStop);
end
